@@ -5,7 +5,7 @@ from unittest.mock import Mock, patch
 import pytest
 from pip._vendor.packaging.specifiers import SpecifierSet
 from pip._vendor.packaging.tags import Tag
-from pkg_resources import parse_version
+from pip._vendor.packaging.version import parse as parse_version
 
 import pip._internal.utils.compatibility_tags
 from pip._internal.exceptions import BestVersionAlreadyInstalled, DistributionNotFound
@@ -79,36 +79,45 @@ def test_incorrect_case_file_index(data: TestData) -> None:
 
 
 @pytest.mark.network
-def test_finder_detects_latest_already_satisfied_find_links(data: TestData) -> None:
+@pytest.mark.parametrize("use_deprecated_html5lib", [False, True])
+def test_finder_detects_latest_already_satisfied_find_links(
+    data: TestData, use_deprecated_html5lib: bool
+) -> None:
     """Test PackageFinder detects latest already satisfied using find-links"""
     req = install_req_from_line("simple", None)
     # the latest simple in local pkgs is 3.0
     latest_version = "3.0"
     satisfied_by = Mock(
         location="/path",
-        parsed_version=parse_version(latest_version),
-        version=latest_version,
+        version=parse_version(latest_version),
     )
     req.satisfied_by = satisfied_by
-    finder = make_test_finder(find_links=[data.find_links])
+    finder = make_test_finder(
+        find_links=[data.find_links], use_deprecated_html5lib=use_deprecated_html5lib
+    )
 
     with pytest.raises(BestVersionAlreadyInstalled):
         finder.find_requirement(req, True)
 
 
 @pytest.mark.network
-def test_finder_detects_latest_already_satisfied_pypi_links() -> None:
+@pytest.mark.parametrize("use_deprecated_html5lib", [False, True])
+def test_finder_detects_latest_already_satisfied_pypi_links(
+    use_deprecated_html5lib: bool,
+) -> None:
     """Test PackageFinder detects latest already satisfied using pypi links"""
     req = install_req_from_line("initools", None)
     # the latest initools on PyPI is 0.3.1
     latest_version = "0.3.1"
     satisfied_by = Mock(
         location="/path",
-        parsed_version=parse_version(latest_version),
-        version=latest_version,
+        version=parse_version(latest_version),
     )
     req.satisfied_by = satisfied_by
-    finder = make_test_finder(index_urls=["http://pypi.org/simple/"])
+    finder = make_test_finder(
+        index_urls=["http://pypi.org/simple/"],
+        use_deprecated_html5lib=use_deprecated_html5lib,
+    )
 
     with pytest.raises(BestVersionAlreadyInstalled):
         finder.find_requirement(req, True)
@@ -185,8 +194,7 @@ class TestWheel:
         latest_version = "1.0"
         satisfied_by = Mock(
             location="/path",
-            parsed_version=parse_version(latest_version),
-            version=latest_version,
+            version=parse_version(latest_version),
         )
         req.satisfied_by = satisfied_by
         finder = make_test_finder(find_links=[data.find_links])
